;-- TIME_STAMP   2021-01-19 16:55:08

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;					 ;;
;;  _XInput UDF for use with AutoIt Beta ;;
;;					                     ;;
;;	Created by Oxin8		             ;;
;;      email: xoxinx@gmail.com		     ;;
;;					                     ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Function _XInputInit()
;	Opens the dll
;	returns - a handle to open dll

Global $g_hXInputDll

Func _XInputInit()
	OnAutoItExitRegister('_XInputExit')
	$g_hXInputDll = DllOpen("XInput9_1_0.dll")
	If @error Then Exit -1
EndFunc   ;==>_XInputInit

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; Function _XInputVibrate($g_hXInputDll, $lmotor, $rmotor)
;	Sets the controller's vibration for the left and right motors
;	$g_hXInputDll - dll handle returned by _XInputInit()
;	$lmotor - desired left motor power level. 0 being none and 100 being full
;	$rmotor - desired left motor power level. 0 being none and 100 being full
;	returns - nothing

Func _XInputVibrate($ilMotor, $irMotor)
	Local $tXInputvibration = DllStructCreate("short;short")
	DllStructSetData($tXInputvibration, 1, Round(($ilMotor / 100) * 32767))
	DllStructSetData($tXInputvibration, 2, Round(($irMotor / 100) * 32767))
	DllCall($g_hXInputDll, "long", "XInputSetState", "long", 0, "ptr", DllStructGetPtr($tXInputvibration))
EndFunc   ;==>_XInputVibrate

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; Function _XInputGetInput($g_hXInputDll)
;	Returns an array of the current controller input
;	$g_hXInputDll - dll handle returned by _XInputInit()
;	returns - an array with info (see below)
;		1-PacketNumber  - indicates change
;		2-Buttons
;		3-Left Trigger  - 0 to 255
;		4-Right Trigger - 0 to 255
;		5-Left X        - -32768 to 32767
;		6-Left Y        - -32768 to 32767
;		7-Right X       - -32768 to 32767
;		8-Right Y       - -32768 to 32767

Func _XInputGetInput()
	Local $aTemp[9]
	Local $tXInputgamepad = DllStructCreate("dword;short;ubyte;ubyte;short;short;short;short")
	If Not DllCall($g_hXInputDll, "long", "XInputGetState", "long", 0, "ptr", DllStructGetPtr($tXInputgamepad)) Then Return SetError(1, 0, False)
	For $i = 1 To 8
		$aTemp[$i] = DllStructGetData($tXInputgamepad, $i)
	Next
	Return $aTemp
EndFunc   ;==>_XInputGetInput

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; Function _XInputButtons($iButtons)
;	Returns an array indicating the pressed buttons
;	$iButtons - use the second element of the array returned by _XInputGetInput
;	returns   - an array of pressed buttons. 1 indicating pressed
;
;	   Array    Element
;               v
; Y    -32768	14
; X     16384   13
; B      8192   12
; A      4096   11
; RButton 512   10
; LButton 256    9
; RJoy    128    8
; LJoy     64    7
; Back     32    6
; Start    16    5
; Right     8    4
; Left      4    3
; Down      2    2
; Up        1    1

Func _XInputButtons($iButtons)
	Local $aPressed[15]
	If $iButtons < 0 Then
		$iButtons += 32768
		$aPressed[14] = 1
	EndIf
	For $i = 14 To 12 Step -1
		If $iButtons >= (2 ^ $i) Then
			$iButtons -= (2 ^ $i)
			$aPressed[($i - 1)] = 1
		EndIf
	Next
	For $i = 9 To 0 Step -1
		If $iButtons >= (2 ^ $i) Then
			$iButtons -= (2 ^ $i)
			$aPressed[($i + 1)] = 1
		EndIf
	Next
	Return $aPressed
EndFunc   ;==>_XInputButtons

Func _XInputExit()
	DllClose($g_hXInputDll)
EndFunc   ;==>_XInputExit
